@@ -1,25 +1,25 @@
 
 module Config
 
-  RUBY_VERSION == "1.6.5" or
-    raise "ruby lib version (1.6.5) doesn't match executable version (#{RUBY_VERSION})"
+  RUBY_VERSION == "1.6.6" or
+    raise "ruby lib version (1.6.6) doesn't match executable version (#{RUBY_VERSION})"
 
 # This file was created by configrb when ruby was built. Any changes
 # made to this file will be lost the next time ruby is built.
   DESTDIR = '' if not defined? DESTDIR
   CONFIG = {}
   TOPDIR = File.dirname(__FILE__).sub!(%r'/lib/ruby/1\.6/i386\-mingw32\Z', '')
-  CONFIG["srcdir"] = "/pub/ruby/16/dist/ruby-1.6.5"
+  CONFIG["srcdir"] = "/pub/ruby/16/dist/ruby-1.6.6"
   CONFIG["prefix"] = (TOPDIR || DESTDIR + "/usr/local")
   CONFIG["ruby_install_name"] = "ruby"
   CONFIG["EXEEXT"] = ".exe"
   CONFIG["SHELL"] = "/bin/sh"
-  CONFIG["CFLAGS"] = "-DNT -D__MSVCRT__ -Os "
+  CONFIG["CFLAGS"] = "-DNT -D__MSVCRT__ -Os"
   CONFIG["CPPFLAGS"] = ""
   CONFIG["CXXFLAGS"] = ""
   CONFIG["FFLAGS"] = ""
   CONFIG["LDFLAGS"] = ""
-  CONFIG["LIBS"] = "-lwsock32 -lmsvcrt "
+  CONFIG["LIBS"] = "-lwsock32 -lmsvcrt"
   CONFIG["exec_prefix"] = "$(prefix)"
   CONFIG["bindir"] = "$(exec_prefix)/bin"
   CONFIG["sbindir"] = "$(exec_prefix)/sbin"
@@ -35,7 +35,7 @@ module Config
   CONFIG["mandir"] = "$(prefix)/man"
   CONFIG["MAJOR"] = "1"
   CONFIG["MINOR"] = "6"
-  CONFIG["TEENY"] = "5"
+  CONFIG["TEENY"] = "6"
   CONFIG["host"] = "i686-pc-cygwin"
   CONFIG["host_alias"] = "i686-pc-cygwin"
   CONFIG["host_cpu"] = "i686"
@@ -51,8 +51,8 @@ module Config
   CONFIG["build_cpu"] = "i586"
   CONFIG["build_vendor"] = "pc"
   CONFIG["build_os"] = "linux-gnu"
-  CONFIG["CC"] = "gcc -bmingw"
-  CONFIG["CPP"] = "gcc -bmingw -E"
+  CONFIG["CC"] = "gcc -mno-cygwin"
+  CONFIG["CPP"] = "gcc -mno-cygwin -E"
   CONFIG["YACC"] = "bison -y"
   CONFIG["RANLIB"] = "ranlib"
   CONFIG["AR"] = "ar"
@@ -70,15 +70,15 @@ module Config
   CONFIG["DLDFLAGS"] = ""
   CONFIG["STATIC"] = ""
   CONFIG["CCDLFLAGS"] = "-DIMPORT"
-  CONFIG["LDSHARED"] = "dllwrap --target=mingw32 --as=as --dlltool-name=dlltool --driver-name=gcc -bmingw --export-all -s"
+  CONFIG["LDSHARED"] = "dllwrap --target=mingw32 --as=as --dlltool-name=dlltool --driver-name=gcc -mno-cygwin --export-all -s"
   CONFIG["DLEXT"] = "so"
   CONFIG["DLEXT2"] = "dll"
   CONFIG["STRIP"] = "strip"
   CONFIG["EXTSTATIC"] = ""
   CONFIG["setup"] = "Setup"
-  CONFIG["MINIRUBY"] = "ruby -I/pub/ruby/16/dist/ruby-1.6.5/mobj -rfake"
+  CONFIG["MINIRUBY"] = "ruby -I/pub/ruby/16/dist/ruby-1.6.6/mobj -rfake"
   CONFIG["PREP"] = "fake.rb"
-  CONFIG["LIBRUBY_LDSHARED"] = "dllwrap --target=mingw32 --as=as --dlltool-name=dlltool --driver-name=gcc -bmingw --export-all -s"
+  CONFIG["LIBRUBY_LDSHARED"] = "dllwrap --target=mingw32 --as=as --dlltool-name=dlltool --driver-name=gcc -mno-cygwin --export-all -s"
   CONFIG["LIBRUBY_DLDFLAGS"] = "--dllname=$@ --output-lib=$(LIBRUBY) --add-stdcall-alias --def=$(RUBYDEF)"
   CONFIG["RUBY_INSTALL_NAME"] = "ruby"
   CONFIG["RUBY_SO_NAME"] = "mingw32-$(RUBY_INSTALL_NAME)16"
@@ -93,13 +93,13 @@ module Config
   CONFIG["MAINLIBS"] = ""
   CONFIG["arch"] = "i386-mingw32"
   CONFIG["sitedir"] = "$(prefix)/lib/ruby/site_ruby"
-  CONFIG["configure_args"] = "--target=i386-mingw32 --host=i686-pc-cygwin --build=i586-pc-linux-gnu --includedir=/usr/local/mingw/include --enable-shared --enable-tcltk_stubs --with-tcllib=tclstub83 --with-tklib=tkstub83 --with-opt-dir=/usr/local/mingw"
+  CONFIG["configure_args"] = "--srcdir=.. --target=i386-mingw32 --host=i686-pc-cygwin --build=i586-pc-linux-gnu --includedir=/usr/local/mingw/include --enable-shared --cache=../../config.cache-mingw --enable-tcltk_stubs --with-tcllib=tclstub83 --with-tklib=tkstub83 --with-opt-dir=/usr/local/mingw"
   CONFIG["ruby_version"] = "$(MAJOR).$(MINOR)"
   CONFIG["rubylibdir"] = "$(libdir)/ruby/$(ruby_version)"
   CONFIG["archdir"] = "$(rubylibdir)/$(arch)"
   CONFIG["sitelibdir"] = "$(sitedir)/$(ruby_version)"
   CONFIG["sitearchdir"] = "$(sitelibdir)/$(arch)"
-  CONFIG["compile_dir"] = "/pub/ruby/16/dist/ruby-1.6.5/mobj"
+  CONFIG["compile_dir"] = "/pub/ruby/16/dist/ruby-1.6.6/mobj"
   MAKEFILE_CONFIG = {}
   CONFIG.each{|k,v| MAKEFILE_CONFIG[k] = v.dup}
   def Config::expand(val)
