@@ -1,267 +1,392 @@
 =begin
 
-= net/http.rb version 1.1.34
+= net/http.rb version 1.1.37
 
-written by Minero Aoki <aamine@dp.u-netsurf.ne.jp>
+Copyright (c) 1999-2001 Yukihiro Matsumoto
 
+written & maintained by Minero Aoki <aamine@loveruby.net>
 This file is derived from "http-access.rb".
 
-This program is free software.
-You can distribute/modify this program under
-the terms of the Ruby Distribute License.
+This program is free software. You can re-distribute and/or
+modify this program under the same terms as Ruby itself,
+Ruby Distribute License or GNU General Public License.
 
-Japanese version of this document is in "net" full package.
-You can get it from RAA
-(Ruby Application Archive: http://www.ruby-lang.org/en/raa.html).
+NOTE: You can find Japanese version of this document in
+the doc/net directory of the standard ruby interpreter package.
 
+== What is this module?
 
-== http.rb version 1.2 features
+This module provide your program the functions to access WWW
+documents via HTTP, Hyper Text Transfer Protocol version 1.1.
+For details of HTTP, refer [RFC2616]
+((<URL:http://www.ietf.org/rfc/rfc2616.txt>)).
 
-You can use 1.2 features by calling HTTP.version_1_2. And
-calling Net::HTTP.version_1_1 allows to use 1.1 features.
+== Examples
 
-  # example
-  HTTP.start {|http1| ...(http1 has 1.1 features)... }
+=== Getting Document From Server
 
-  HTTP.version_1_2
-  HTTP.start {|http2| ...(http2 has 1.2 features)... }
+Be care to ',' (comma) putted after "response".
+This is required for compatibility.
 
-  HTTP.version_1_1
-  HTTP.start {|http3| ...(http3 has 1.1 features)... }
+    require 'net/http'
+    Net::HTTP.start( 'some.www.server', 80 ) {|http|
+      response , = http.get('/index.html')
+      puts response.body
+    }
 
-Changes are:
+(shorter version)
 
-  * HTTP#get, head, post does not raise ProtocolError
-  * HTTP#get, head, post returns only one object, a HTTPResponse object
-  * HTTPResponseReceiver is joined into HTTPResponse
-  * request object: HTTP::Get, Head, Post; and HTTP#request(req)
+    require 'net/http'
+    Net::HTTP.get_print 'some.www.server', '/index.html'
 
-WARNING: These features are not definite yet.
-They will change without notice!
+=== Posting Form Data
 
+    require 'net/http'
+    Net::HTTP.start( 'some.www.server', 80 ) {|http|
+        response , = http.post( '/cgi-bin/any.rhtml',
+                                'querytype=subject&target=ruby' )
+    }
 
-== class HTTP
+=== Accessing via Proxy
+
+Net::HTTP.Proxy() creates http proxy class. It has same
+methods of Net::HTTP but its instances always connect to
+proxy, instead of given host.
+
+    require 'net/http'
+
+    $proxy_addr = 'your.proxy.addr'
+    $proxy_port = 8080
+          :
+    Net::HTTP::Proxy($proxy_addr, $proxy_port).start( 'some.www.server' ) {|http|
+      # always connect to your.proxy.addr:8080
+          :
+    }
+
+Since Net::HTTP.Proxy() returns Net::HTTP itself when $proxy_addr is nil,
+there's no need to change code if there's proxy or not.
+
+=== Redirect
+
+    require 'net/http'
+    Net::HTTP.version_1_1
+
+    host = 'www.ruby-lang.org'
+    path = '/'
+    begin
+      Net::HTTP.start( host, 80 ) {|http|
+	response , = http.get(path)
+        print response.body
+      }
+    rescue Net::ProtoRetriableError => err
+      if m = %r<http://([^/]+)>.match( err.response['location'] ) then
+	host = m[1].strip
+	path = m.post_match
+	retry
+      end
+    end
+
+NOTE: This code is using ad-hoc way to extract host name, but in future
+URI class will be included in ruby standard library.
+
+=== Basic Authentication
+
+    require 'net/http'
+
+    Net::HTTP.start( 'auth.some.domain' ) {|http|
+        response , = http.get( '/need-auth.cgi',
+                'Authentication' => ["#{account}:#{password}"].pack('m').strip )
+        print response.body
+    }
+
+In version 1.2 (Ruby 1.7 or later), you can write like this:
+
+    require 'net/http'
+
+    req = Net::HTTP::Get.new('/need-auth.cgi')
+    req.basic_auth 'account', 'password'
+    Net::HTTP.start( 'auth.some.domain' ) {|http|
+        response = http.request(req)
+        print response.body
+    }
+
+== Switching Net::HTTP versions
+
+You can use old Net::HTTP (in Ruby 1.6) features by calling
+HTTP.version_1_1. And calling Net::HTTP.version_1_2 allows
+you to use 1.2 features again.
+
+    # example
+    Net::HTTP.start {|http1| ...(http1 has 1.2 features)... }
+
+    Net::HTTP.version_1_1
+    Net::HTTP.start {|http2| ...(http2 has 1.1 features)... }
+
+    Net::HTTP.version_1_2
+    Net::HTTP.start {|http3| ...(http3 has 1.2 features)... }
+
+Yes, this is not thread-safe.
+
+== class Net::HTTP
 
 === Class Methods
 
-: new( address = 'localhost', port = 80, proxy_addr = nil, proxy_port = nil )
-  creates a new Net::HTTP object.
-  If proxy_addr is given, this method is equals to
-  Net::HTTP::Proxy(proxy_addr,proxy_port).
+: new( address, port = 80, proxy_addr = nil, proxy_port = nil )
+    creates a new Net::HTTP object.
+    If proxy_addr is given, creates an Net::HTTP object with proxy support.
 
-: start( address = 'localhost', port = 80, proxy_addr = nil, proxy_port = nil )
-: start( address = 'localhost', port = 80, proxy_addr = nil, proxy_port = nil ) {|http| .... }
-  is equals to
-    Net::HTTP.new( address, port, proxy_addr, proxy_port ).start(&block)
+: start( address, port = 80, proxy_addr = nil, proxy_port = nil )
+: start( address, port = 80, proxy_addr = nil, proxy_port = nil ) {|http| .... }
+    is equals to
 
-: Proxy( address, port )
-  creates a HTTP proxy class.
-  Arguments are address/port of proxy host.
-  You can replace HTTP class by this proxy class.
+        Net::HTTP.new(address, port, proxy_addr, proxy_port).start(&block)
 
-    # example
-    proxy_http = HTTP::Proxy( 'proxy.foo.org', 8080 )
-      :
-    proxy_http.start( 'www.ruby-lang.org' ) do |http|
-      # connecting proxy.foo.org:8080
-      :
-    end
+: get( address, path, port = 80 )
+    gets entity body from path and returns it.
+    return value is a String.
+
+: get_print( address, path, port = 80 )
+    gets entity body from path and output it to $stdout.
+
+: Proxy( address, port = 80 )
+    creates a HTTP proxy class.
+    Arguments are address/port of proxy host.
+    You can replace HTTP class with created proxy class.
+
+    If ADDRESS is nil, this method returns self (Net::HTTP).
+
+        # example
+        proxy_class = Net::HTTP::Proxy( 'proxy.foo.org', 8080 )
+                        :
+        proxy_class.start( 'www.ruby-lang.org' ) {|http|
+            # connecting proxy.foo.org:8080
+                        :
+        }
 
 : proxy_class?
-  If self is HTTP, false.
-  If self is a class which was created by HTTP::Proxy(), true.
+    If self is HTTP, false.
+    If self is a class which was created by HTTP::Proxy(), true.
 
 : port
-  HTTP default port (80).
+    default HTTP port (80).
 
-
-=== Methods
+=== Instance Methods
 
 : start
 : start {|http| .... }
-  creates a new Net::HTTP object and starts HTTP session.
+    creates a new Net::HTTP object and starts HTTP session.
 
-  When this method is called with block, gives a HTTP object to block
-  and close HTTP session after returning from the block.
+    When this method is called with block, gives a HTTP object to block
+    and close the HTTP session after block call finished.
+
+: active?
+    true if HTTP session is started.
+
+: address
+    the address to connect
+
+: port
+    the port number to connect
+
+: open_timeout
+: open_timeout=(n)
+    seconds to wait until connection is opened.
+    If HTTP object cannot open a conection in this seconds,
+    it raises TimeoutError exception.
+
+: read_timeout
+: read_timeout=(n)
+    seconds to wait until reading one block (by one read(1) call).
+    If HTTP object cannot open a conection in this seconds,
+    it raises TimeoutError exception.
+
+: finish
+    finishes HTTP session.
+    If HTTP session had not started, raises an IOError.
 
 : proxy?
-  true if self is a HTTP proxy class
+    true if self is a HTTP proxy class
 
 : proxy_address
-  address of proxy host. If self is not a proxy, nil.
+    address of proxy host. If self does not use a proxy, nil.
 
 : proxy_port
-  port number of proxy host. If self is not a proxy, nil.
+    port number of proxy host. If self does not use a proxy, nil.
 
 : get( path, header = nil, dest = '' )
 : get( path, header = nil ) {|str| .... }
-  get data from "path" on connecting host.
-  "header" must be a Hash like { 'Accept' => '*/*', ... }.
-  Data is written to "dest" by using "<<" method.
-  This method returns HTTPResponse object, and "dest".
+    gets data from PATH on the connecting host.
+    HEADER must be a Hash like { 'Accept' => '*/*', ... }.
+    Response body is written into DEST by using "<<" method.
 
-    # example
-    response, body = http.get( '/index.html' )
+    This method returns Net::HTTPResponse object.
 
-  If called with block, give a part String of entity body.
+    If called with block, gives entity body little by little
+    to the block (as String).
 
-  Note:
-  If status is not 2xx(success), ProtocolError exception is
-  raised. At that time, you can get HTTPResponse object from 
-  exception object. (same in head/post)
+    In version 1.1, this method might raises exception for also
+    3xx (redirect). On the case you can get a HTTPResponse object
+    by "anException.response".
 
-    # example
-    begin
-      response, body = http.get( '/index.html' )
-    rescue Net::ProtoRetriableError => err
-      response = err.data
-      ...
-    end
+    In version 1.2, this method never raises exception.
+
+        # version 1.1 (bundled with Ruby 1.6)
+        response, body = http.get( '/index.html' )
+
+        # version 1.2 (bundled with Ruby 1.7 or later)
+        response = http.get( '/index.html' )
+
+        # compatible in both version
+        response , = http.get( '/index.html' )
+        response.body
+        
+        # using block
+        File.open( 'save.txt', 'w' ) {|f|
+            http.get( '/~foo/', nil ) do |str|
+              f.write str
+            end
+        }
+        # same effect
+        File.open( 'save.txt', 'w' ) {|f|
+            http.get '/~foo/', nil, f
+        }
 
 : head( path, header = nil )
-  gets only header from "path" on connecting host.
-  "header" is a Hash like { 'Accept' => '*/*', ... }.
-  This method returns a HTTPResponse object.
-  You can http header from this object like:
+    gets only header from PATH on the connecting host.
+    HEADER is a Hash like { 'Accept' => '*/*', ... }.
 
-    response['content-length']   #-> '2554'
-    response['content-type']     #-> 'text/html'
-    response['Content-Type']     #-> 'text/html'
-    response['CoNtEnT-tYpe']     #-> 'text/html'
+    This method returns a Net::HTTPResponse object.
+
+    In version 1.1, this method might raises exception for also
+    3xx (redirect). On the case you can get a HTTPResponse object
+    by "anException.response".
+
+        response = nil
+        Net::HTTP.start( 'some.www.server', 80 ) {|http|
+            response = http.head( '/index.html' )
+        }
+        p response['content-type']
 
 : post( path, data, header = nil, dest = '' )
 : post( path, data, header = nil ) {|str| .... }
-  posts "data" (must be String now) to "path".
-  If the body exists, also gets entity body.
-  Data is written to "dest" by using "<<" method.
-  "header" must be a Hash like { 'Accept' => '*/*', ... }.
-  This method returns HTTPResponse object and "dest".
+    posts "data" (must be String) to "path".
+    If the body exists, also gets entity body.
+    Response body is written into "dest" by using "<<" method.
+    "header" must be a Hash like { 'Accept' => '*/*', ... }.
+    This method returns Net::HTTPResponse object.
 
-  If called with block, gives a part of entity body string.
+    If called with block, gives a part of entity body string.
+
+    In version 1.1, this method might raises exception for also
+    3xx (redirect). On the case you can get a HTTPResponse object
+    by "anException.response".
+
+        # version 1.1
+        response, body = http.post( '/cgi-bin/search.rb', 'querytype=subject&target=ruby' )
+        # version 1.2
+        response = http.post( '/cgi-bin/search.rb', 'querytype=subject&target=ruby' )
+        # compatible for both version
+        response , = http.post( '/cgi-bin/search.rb', 'querytype=subject&target=ruby' )
+
+        # using block
+        File.open( 'save.html', 'w' ) {|f|
+            http.post( '/cgi-bin/search.rb', 'querytype=subject&target=ruby' ) do |str|
+              f.write str
+            end
+        }
+        # same effect
+        File.open( 'save.html', 'w' ) {|f|
+            http.post '/cgi-bin/search.rb', 'querytype=subject&target=ruby', nil, f
+        }
 
 : get2( path, header = nil )
-: get2( path, header = nil ) {|recv| .... }
-  send GET request for "path".
-  "header" must be a Hash like { 'Accept' => '*/*', ... }.
-  If this method is called with block, one gives
-  a HTTPResponseReceiver object to block.
+: get2( path, header = nil ) {|response| .... }
+    gets entity from PATH. This method returns a HTTPResponse object.
 
-    # example
-    http.get2( '/index.html' ) do |recv|
-      # "recv" is a HTTPResponseReceiver object
-      recv.header
-      recv.body
-    end
+    When called with block, keep connection while block is executed
+    and gives a HTTPResponse object to the block.
 
-    # another way
-    response = http.get2( '/index.html' )
-    response['content-type']
-    response.body
+    This method never raise any ProtocolErrors.
 
-    # this is wrong
-    http.get2( '/index.html' ) do |recv|
-      print recv.response.body   # body is not read yet!!!
-    end
+        # example
+        response = http.get2( '/index.html' )
+        p response['content-type']
+        puts response.body          # body is already read
 
-    # but this is ok
-    http.get2( '/index.html' ) do |recv|
-      recv.body                  # read body and set recv.response.body
-      print recv.response.body   # ref
-    end
+        # using block
+        http.get2( '/index.html' ) {|response|
+            p response['content-type']
+            response.read_body do |str|   # read body now
+              print str
+            end
+        }
 
-: head2( path, header = nil )
-: head2( path, header = nil ) {|recv| .... }
-  send HEAD request for "path".
-  "header" must be a Hash like { 'Accept' => 'text/html', ... }.
-  The difference between "head" method is that
-  "head2" does not raise exceptions.
+: post2( path, header = nil )
+: post2( path, header = nil ) {|response| .... }
+    posts data to PATH. This method returns a HTTPResponse object.
 
-  If this method is called with block, one gives
-  a HTTPResponseReceiver object to block.
+    When called with block, gives a HTTPResponse object to the block
+    before reading entity body, with keeping connection.
 
-    # example
-    response = http.head2( '/index.html' )
+        # example
+        response = http.post2( '/cgi-bin/nice.rb', 'datadatadata...' )
+        p response.status
+        puts response.body          # body is already read
 
-    # another way
-    http.head2( '/index.html' ) do |recv|
-      recv.response
-    end
-
-: post2( path, data, header = nil )
-: post2( path, data, header = nil ) {|recv| .... }
-  posts "data" (must be String now) to "path".
-  "header" must be a Hash like { 'Accept' => '*/*', ... }.
-  If this method is called with block, one gives
-  a HTTPResponseReceiver object to block.
-
-    # example
-    http.post2( '/anycgi.rb', 'data data data...' ) do |recv|
-      # "recv" is a HTTPResponseReceiver object
-      recv.header
-      recv.body
-    end
-
-    # another way
-    response = http.post2( '/anycgi.rb', 'important data' )
-    response['content-type']
-    response.body
+        # using block
+        http.post2( '/cgi-bin/nice.rb', 'datadatadata...' ) {|response|
+            p response.status
+            p response['content-type']
+            response.read_body do |str|   # read body now
+              print str
+	    end
+        }
 
 
-== class HTTPResponse
+== class Net::HTTPResponse
 
-HTTP response object.
-All "key" is case-insensitive.
+HTTP response class. This class wraps response header and entity.
+All arguments named KEY is case-insensitive.
 
-=== Methods
-
-: body
-  the entity body (String).
+=== Instance Methods
 
 : self[ key ]
-  returns header field for "key".
-  for HTTP, value is a string like 'text/plain'(for Content-Type),
-  '2045'(for Content-Length), 'bytes 0-1023/10024'(for Content-Range).
-  If there's some fields which has same name, they are joined with ','.
+    returns the header field corresponding to the case-insensitive key.
+    For example, a key of "Content-Type" might return "text/html".
+    A key of "Content-Length" might do "2045".
+
+    More than one fields which has same names are joined with ','.
 
 : self[ key ] = val
-  set field value for "key".
+    sets the header field corresponding to the case-insensitive key.
 
 : key?( key )
-  true if key exists
+    true if key exists.
+    KEY is case insensitive.
 
 : each {|name,value| .... }
-  iterates for each field name and value pair.
+    iterates for each field name and value pair.
+
+: canonical_each {|name,value| .... }
+    iterates for each "canonical" field name and value pair.
 
 : code
-  HTTP result code string. For example, '302'.
+    HTTP result code string. For example, '302'.
 
 : message
-  HTTP result message. For example, 'Not Found'.
-
-
-== class HTTPResponseReceiver
-
-=== Methods
-
-: header
-: response
-  HTTPResponse object
+    HTTP result message. For example, 'Not Found'.
 
 : read_body( dest = '' )
-  reads entity body into DEST by calling "<<" method and
-  returns DEST.
+    gets entity body and write it into DEST using "<<" method.
+    If this method is called twice or more, nothing will be done
+    and returns first DEST.
 
-: read_body {|string| ... }
-  reads entity body little by little and gives it to block
-  until entity ends.
+: read_body {|str| .... }
+    gets entity body little by little and pass it to block.
 
 : body
-: entity
-  entity body. If #read_body is called already, returns its
-  argument DEST. Else returns entity body as String.
-
-  Calling this method any times causes returning same
-  object (does not read entity again).
+    response body. If #read_body has been called, this method returns
+    arg of #read_body DEST. Else gets body as String and returns it.
 
 =end
 
@@ -284,18 +409,13 @@ module Net
 
     protocol_param :port, '80'
 
-
-    def initialize( addr = nil, port = nil )
+    def initialize( addr, port = nil )
       super
 
-      @proxy_address = nil
-      @proxy_port = nil
-
       @curr_http_version = HTTPVersion
       @seems_1_0_server = false
     end
 
-
     private
 
     def conn_command( sock )
@@ -315,48 +435,54 @@ module Net
     class << self
 
       def Proxy( p_addr, p_port = nil )
-        ProxyMod.create_proxy_class( p_addr, p_port || self.port )
+        if p_addr then
+          ProxyMod.create_proxy_class( p_addr, p_port || self.port )
+        else
+          self
+        end
       end
 
       alias orig_new new
 
-      def new( address = nil, port = nil, p_addr = nil, p_port = nil )
+      def new( address, port = nil, p_addr = nil, p_port = nil )
         c = p_addr ? self::Proxy(p_addr, p_port) : self
         i = c.orig_new( address, port )
         setvar i
         i
       end
 
-      def start( address = nil, port = nil, p_addr = nil, p_port = nil, &block )
+      def start( address, port = nil, p_addr = nil, p_port = nil, &block )
         new( address, port, p_addr, p_port ).start( &block )
       end
 
+      @is_proxy_class = false
+      @proxy_addr = nil
+      @proxy_port = nil
+
       def proxy_class?
-        false
+        @is_proxy_class
       end
 
-      def proxy_address
-        nil
-      end
-
-      def proxy_port
-        nil
-      end
+      attr_reader :proxy_address
+      attr_reader :proxy_port
 
     end
 
     def proxy?
-      false
+      type.proxy_class?
     end
 
     def proxy_address
-      nil
+      type.proxy_address
     end
 
     def proxy_port
-      nil
+      type.proxy_port
     end
 
+    alias proxyaddr proxy_address
+    alias proxyport proxy_port
+
     def edit_path( path )
       path
     end
@@ -364,52 +490,22 @@ module Net
 
     module ProxyMod
 
-      class << self
-
-        def create_proxy_class( p_addr, p_port )
-          mod = self
-          klass = Class.new( HTTP )
-          klass.module_eval {
-            include mod
-            @proxy_address = p_addr
-            @proxy_port    = p_port
-          }
-          def klass.proxy_class?
-            true
-          end
-
-          def klass.proxy_address
-            @proxy_address
-          end
-
-          def klass.proxy_port
-            @proxy_port
-          end
-
-          klass
-        end
-
+      def self.create_proxy_class( p_addr, p_port )
+        mod = self
+        klass = Class.new( HTTP )
+        klass.module_eval {
+          include mod
+          @is_proxy_class = true
+          @proxy_address = p_addr
+          @proxy_port    = p_port
+        }
+        klass
       end
 
-      def initialize( addr, port )
-        super
-        @proxy_address = type.proxy_address
-        @proxy_port    = type.proxy_port
-      end
-    
-      attr_reader :proxy_address, :proxy_port
-
-      alias proxyaddr proxy_address
-      alias proxyport proxy_port
-
-      def proxy?
-        true
-      end
-    
       private
     
       def conn_socket( addr, port )
-        super @proxy_address, @proxy_port
+        super proxy_address, proxy_port
       end
 
       def edit_path( path )
@@ -423,7 +519,11 @@ module Net
     # for backward compatibility
     #
 
-    @@newimpl = false
+    if Version < '1.2.0' then   ###noupdate
+      @@newimpl = false
+    else
+      @@newimpl = true
+    end
 
     class << self
 
@@ -435,6 +535,10 @@ module Net
         @@newimpl = false
       end
 
+      def is_version_1_2?
+        @@newimpl
+      end
+
       private
 
       def setvar( obj )
@@ -451,7 +555,7 @@ module Net
 
     public
 
-    def self.def_http_method( nm, hasdest, hasdata )
+    def self.define_http_method_interface( nm, hasdest, hasdata )
       name = nm.id2name.downcase
       cname = nm.id2name
       lineno = __LINE__ + 2
@@ -482,36 +586,46 @@ module Net
       module_eval src, __FILE__, lineno
     end
 
-    def_http_method :Get,  true,  false
-    def_http_method :Head, false, false
-    def_http_method :Post, true,  true
-    def_http_method :Put,  false, true
+    define_http_method_interface :Get,  true,  false
+    define_http_method_interface :Head, false, false
+    define_http_method_interface :Post, true,  true
+    define_http_method_interface :Put,  false, true
 
-    def request( req, *args )
-      common_oper( req ) {
+    def request( req, body = nil, &block )
+      unless active? then
+        start {
+          req['connection'] = 'close'
+          return request(req, body, &block)
+        }
+      end
+        
+      connecting( req ) {
         req.__send__( :exec,
-                @socket, @curr_http_version, edit_path(req.path), *args )
+                @socket, @curr_http_version, edit_path(req.path), body )
         yield req.response if block_given?
       }
       req.response
     end
 
+    def send_request( name, path, body = nil, header = nil )
+      r = ::Net::NetPrivate::HTTPGenericRequest.new(
+              name, (body ? true : false), true,
+              path, header )
+      request r, body
+    end
+
 
     private
 
 
-    def common_oper( req )
-      req['connection'] ||= 'keep-alive'
-      if not @socket then
-        start
-        req['connection'] = 'close'
-      elsif @socket.closed? then
+    def connecting( req )
+      if @socket.closed? then
         re_connect
       end
       if not req.body_exist? or @seems_1_0_server then
         req['connection'] = 'close'
       end
-      req['host'] = addr_port
+      req['host'] = addr_port()
 
       yield req
       req.response.__send__ :terminate
@@ -530,8 +644,6 @@ module Net
         D 'Conn close'
         @socket.close
       end
-
-      req.response
     end
 
     def keep_alive?( req, res )
@@ -564,7 +676,10 @@ module Net
     end
 
     def self.get_print( addr, path, port = nil )
-      print get( addr, path, port )
+      new( addr, port || HTTP.port ).start {|http|
+        http.get path, nil, $stdout
+      }
+      nil
     end
 
 
@@ -719,15 +834,15 @@ module Net
 
         d1 = m[1].to_i
         d2 = m[2].to_i
-        if    m[1] and m[2] then arr.push (d1 .. d2)
-        elsif m[1]          then arr.push (d1 .. -1)
-        elsif          m[2] then arr.push (-d2 .. -1)
+        if    m[1] and m[2] then arr.push(  d1..d2 )
+        elsif m[1]          then arr.push(  d1..-1 )
+        elsif          m[2] then arr.push( -d2..-1 )
         else
           raise HTTPHeaderSyntaxError, 'range is not specified'
         end
       end
 
-      return *arr
+      return arr
     end
 
     def range=( r, fin = nil )
@@ -793,27 +908,27 @@ module Net
     end
 
     def basic_auth( acc, pass )
-      @header['authorization'] =
-              'Basic ' + ["#{acc}:#{pass}"].pack('m').gsub(/\s+/, '')
+      @header['authorization'] = 'Basic ' + ["#{acc}:#{pass}"].pack('m').strip
     end
 
   end
 
-  }
-
 
   ###
   ### request
   ###
 
-  net_private {
-
-  class HTTPRequest
+  class HTTPGenericRequest
 
     include ::Net::NetPrivate::HTTPHeader
 
-    def initialize( path, uhead = nil )
+    def initialize( m, reqbody, resbody, path, uhead = nil )
+      @method = m
+      @request_has_body = reqbody
+      @response_has_body = resbody
       @path = path
+      @response = nil
+
       @header = tmp = {}
       return unless uhead
       uhead.each do |k,v|
@@ -824,11 +939,9 @@ module Net
         tmp[ key ] = v.strip
       end
       tmp['accept'] ||= '*/*'
-
-      @socket = nil
-      @response = nil
     end
 
+    attr_reader :method
     attr_reader :path
     attr_reader :response
 
@@ -836,10 +949,16 @@ module Net
       "\#<#{type}>"
     end
 
-    def body_exist?
-      type::HAS_BODY
+    def request_body_permitted?
+      @request_has_body
     end
 
+    def response_body_permitted?
+      @response_has_body
+    end
+
+    alias body_exist? response_body_permitted?
+
 
     private
 
@@ -847,110 +966,89 @@ module Net
     # write
     #
 
-    def exec( sock, ver, path )
-      ready( sock ) {
-        request ver, path
-      }
-      @response
-    end
-
-    def ready( sock )
-      @response = nil
-      @socket = sock
-      yield
-      @response = get_response
-      @socket = nil
-    end
-
-    def request( ver, path )
-      @socket.writeline sprintf('%s %s HTTP/%s', type::METHOD, path, ver)
-      canonical_each do |k,v|
-        @socket.writeline k + ': ' + v
+    def exec( sock, ver, path, body, &block )
+      if body then
+        check_body_premitted
+        check_arg_b body, block
+        sendreq_with_body sock, ver, path, body, &block
+      else
+        check_arg_n body
+        sendreq_no_body sock, ver, path
       end
-      @socket.writeline ''
+      @response = r = get_response( sock )
+      r
+    end
+
+    def check_body_premitted
+      request_body_permitted? or
+          raise ArgumentError, 'HTTP request body is not premitted'
+    end
+
+    def check_arg_b( data, block )
+      if data and block then
+        raise ArgumentError, 'both of data and block given'
+      end
+      unless data or block then
+        raise ArgumentError, 'str or block required'
+      end
+    end
+
+    def check_arg_n( data )
+      data and raise ArgumentError, "data is not permitted for #{@method}"
+    end
+
+
+    def sendreq_no_body( sock, ver, path )
+      request sock, ver, path
+    end
+
+    def sendreq_with_body( sock, ver, path, body )
+      if block_given? then
+        ac = Accumulator.new
+        yield ac              # must be yield, DO NOT USE block.call
+        data = ac.terminate
+      else
+        data = body
+      end
+      @header['content-length'] = data.size.to_s
+      @header.delete 'transfer-encoding'
+
+      request sock, ver, path
+      sock.write data
+    end
+
+    def request( sock, ver, path )
+      sock.writeline sprintf('%s %s HTTP/%s', @method, path, ver)
+      canonical_each do |k,v|
+        sock.writeline k + ': ' + v
+      end
+      sock.writeline ''
     end
 
     #
     # read
     #
 
-    def get_response
+    def get_response( sock )
       begin
-        resp = read_response
+        resp = ::Net::NetPrivate::HTTPResponse.new_from_socket(sock,
+                                                response_body_permitted?)
       end while ContinueCode === resp
       resp
     end
-
-    def read_response
-      resp = get_resline
-
-      while true do
-        line = @socket.readuntil( "\n", true )   # ignore EOF
-        line.sub!( /\s+\z/, '' )                 # don't use chop!
-        break if line.empty?
-
-        m = /\A([^:]+):\s*/.match( line )
-        m or raise HTTPBadResponse, 'wrong header line format'
-        nm = m[1]
-        line = m.post_match
-        if resp.key? nm then
-          resp[nm] << ', ' << line
-        else
-          resp[nm] = line
-        end
-      end
-
-      resp
-    end
-
-    def get_resline
-      str = @socket.readline
-      m = /\AHTTP(?:\/(\d+\.\d+))?\s+(\d\d\d)\s*(.*)\z/i.match( str )
-      m or raise HTTPBadResponse, "wrong status line: #{str}"
-      httpver = m[1]
-      status  = m[2]
-      discrip = m[3]
-      
-      ::Net::NetPrivate::HTTPResponse.new(
-              status, discrip, @socket, type::HAS_BODY, httpver )
-    end
   
   end
 
 
-  class HTTPRequestWithBody < HTTPRequest
-  
-    private
+  class HTTPRequest < HTTPGenericRequest
 
-    def exec( sock, ver, path, str = nil )
-      check_arg str, block_given?
-
-      if block_given? then
-        ac = Accumulator.new
-        yield ac              # must be yield, DO NOT USE block.call
-        data = ac.terminate
-      else
-        data = str
-      end
-      @header['content-length'] = data.size.to_s
-      @header.delete 'transfer-encoding'
-
-      ready( sock ) {
-        request ver, path
-        @socket.write data
-      }
-      @response
+    def initialize( path, uhead = nil )
+      super type::METHOD,
+            type::REQUEST_HAS_BODY,
+            type::RESPONSE_HAS_BODY,
+            path, uhead
     end
 
-    def check_arg( data, blkp )
-      if data and blkp then
-        raise ArgumentError, 'both of data and block given'
-      end
-      unless data or blkp then
-        raise ArgumentError, 'str or block required'
-      end
-    end
-  
   end
 
 
@@ -964,7 +1062,10 @@ module Net
       @buf.concat s
     end
 
-    alias << write
+    def <<( s )
+      @buf.concat s
+      self
+    end
 
     def terminate
       ret = @buf
@@ -980,23 +1081,27 @@ module Net
   class HTTP
 
     class Get < ::Net::NetPrivate::HTTPRequest
-      HAS_BODY = true
       METHOD = 'GET'
+      REQUEST_HAS_BODY  = false
+      RESPONSE_HAS_BODY = true
     end
 
     class Head < ::Net::NetPrivate::HTTPRequest
-      HAS_BODY = false
       METHOD = 'HEAD'
+      REQUEST_HAS_BODY = false
+      RESPONSE_HAS_BODY = false
     end
 
-    class Post < ::Net::NetPrivate::HTTPRequestWithBody
-      HAS_BODY = true
+    class Post < ::Net::NetPrivate::HTTPRequest
       METHOD = 'POST'
+      REQUEST_HAS_BODY = true
+      RESPONSE_HAS_BODY = true
     end
 
-    class Put < ::Net::NetPrivate::HTTPRequestWithBody
-      HAS_BODY = true
+    class Put < ::Net::NetPrivate::HTTPRequest
       METHOD = 'PUT'
+      REQUEST_HAS_BODY = true
+      RESPONSE_HAS_BODY = true
     end
 
   end
@@ -1065,6 +1170,45 @@ module Net
       '505' => HTTPVersionNotSupported
     }
 
+
+    class << self
+
+      def new_from_socket( sock, hasbody )
+        resp = readnew( sock, hasbody )
+
+        while true do
+          line = sock.readuntil( "\n", true )   # ignore EOF
+          line.sub!( /\s+\z/, '' )                 # don't use chop!
+          break if line.empty?
+
+          m = /\A([^:]+):\s*/.match( line )
+          m or raise HTTPBadResponse, 'wrong header line format'
+          nm = m[1]
+          line = m.post_match
+          if resp.key? nm then
+            resp[nm] << ', ' << line
+          else
+            resp[nm] = line
+          end
+        end
+
+        resp
+      end
+
+      private
+
+      def readnew( sock, hasbody )
+        str = sock.readline
+        m = /\AHTTP(?:\/(\d+\.\d+))?\s+(\d\d\d)\s*(.*)\z/in.match( str )
+        m or raise HTTPBadResponse, "wrong status line: #{str}"
+        discard, httpv, stat, desc = *m.to_a
+        
+        new( stat, desc, sock, hasbody, httpv )
+      end
+
+    end
+
+
     def initialize( stat, msg, sock, be, hv )
       code = CODE_TO_OBJ[stat] ||
              CODE_CLASS_TO_OBJ[stat[0,1]] ||
@@ -1086,7 +1230,7 @@ module Net
     end
 
     def value
-      SuccessCode === self or error! self
+      SuccessCode === self or error!
     end
 
 
@@ -1094,12 +1238,12 @@ module Net
     # header (for backward compatibility)
     #
 
-    def read_header
+    def response
       self
     end
 
-    alias header read_header
-    alias response read_header
+    alias header response
+    alias read_header response
 
     #
     # body
